@@ -1,7 +1,7 @@
 package Log::ger;
 
-our $DATE = '2017-06-30'; # DATE
-our $VERSION = '0.011'; # VERSION
+our $DATE = '2017-07-02'; # DATE
+our $VERSION = '0.012'; # VERSION
 
 #IFUNBUILT
 # # use strict;
@@ -235,11 +235,13 @@ sub init_target {
             push @rn, @{ $routine_names->{log_methods} || [] };
             push @rn, @{ $routine_names->{logml_methods} || [] };
         }
+        my $mllogger0;
         for my $rn (@rn) {
             my ($rname, $lname) = @$rn;
             my $lnum = $Levels{$lname} if defined $lname;
-
             my $routine_name_is_ml = !defined($lname);
+
+            my $logger;
             my ($logger0, $logger0_is_ml);
             $_logger_is_null = 0;
             for my $phase (qw/create_logml_routine create_log_routine/) {
@@ -247,9 +249,18 @@ sub init_target {
                 local $hook_args{level} = $lnum;
                 local $hook_args{str_level} = $lname;
                 $logger0_is_ml = $phase eq 'create_logml_routine';
+                if ($mllogger0) {
+                    # we reuse the same multilevel logger0 for all log routines,
+                    # since it can handle different levels
+                    $logger0 = $mllogger0;
+                    last;
+                }
                 $logger0 = run_hooks(
                     $phase, \%hook_args, 1, $target, $target_arg)
                     or next;
+                if ($logger0_is_ml) {
+                    $mllogger0 = $logger0;
+                }
                 last;
             }
             # this can happen if there is no create_logml_routine hook but
@@ -261,7 +272,6 @@ sub init_target {
 
             require Log::ger::Util if !$logger0_is_ml && $routine_name_is_ml;
 
-            my $logger;
             {
                 if ($_logger_is_null) {
                     # if logger is a null logger (sub {0}) we don't need to
@@ -275,41 +285,43 @@ sub init_target {
                     if ($layouter) {
                         if ($logger0_is_ml) {
                             if ($routine_name_is_ml) {
-                                if ($object) { $logger = sub { shift; my $lvl=shift; $logger0->($init_args, $lvl,  $layouter->($formatter->(@_), $init_args, $lnum, $lname)) };
-                                } else {       $logger = sub {        my $lvl=shift; $logger0->($init_args, $lvl,  $layouter->($formatter->(@_), $init_args, $lnum, $lname)) }; }
+                                if ($object) { $logger = sub { shift; my $lnum=shift; my $lname = Log::ger::Util::string_level($lnum);
+                                                                                      $logger0->($init_args, $lnum, $layouter->($formatter->(@_), $init_args, $lnum, $lname)) };
+                                } else {       $logger = sub {        my $lnum=shift; my $lname = Log::ger::Util::string_level($lnum);
+                                                                                      $logger0->($init_args, $lnum, $layouter->($formatter->(@_), $init_args, $lnum, $lname)) }; }
                             } else { # routine name not multiple-level
-                                if ($object) { $logger = sub { shift;                $logger0->($init_args, $lnum, $layouter->($formatter->(@_), $init_args, $lnum, $lname)) };
-                                } else {       $logger = sub {                       $logger0->($init_args, $lnum, $layouter->($formatter->(@_), $init_args, $lnum, $lname)) }; }
+                                if ($object) { $logger = sub { shift;                 $logger0->($init_args, $lnum, $layouter->($formatter->(@_), $init_args, $lnum, $lname)) };
+                                } else {       $logger = sub {                        $logger0->($init_args, $lnum, $layouter->($formatter->(@_), $init_args, $lnum, $lname)) }; }
                             }
                         } else { # logger0 not multiple-level
                             if ($routine_name_is_ml) {
                                 if ($object) { $logger = sub { shift; return 0 if Log::ger::Util::numeric_level(shift) > $Current_Level;
-                                                                                     $logger0->($init_args,        $layouter->($formatter->(@_), $init_args, $lnum, $lname)) };
+                                                                                      $logger0->($init_args,        $layouter->($formatter->(@_), $init_args, $lnum, $lname)) };
                                 } else {       $logger = sub {        return 0 if Log::ger::Util::numeric_level(shift) > $Current_Level;
-                                                                                 $logger0->($init_args,        $layouter->($formatter->(@_), $init_args, $lnum, $lname)) }; }
+                                                                                      $logger0->($init_args,        $layouter->($formatter->(@_), $init_args, $lnum, $lname)) }; }
                             } else { # routine name not multiple-level
-                                if ($object) { $logger = sub { shift;                $logger0->($init_args,        $layouter->($formatter->(@_), $init_args, $lnum, $lname)) };
-                                } else {       $logger = sub {                       $logger0->($init_args,        $layouter->($formatter->(@_), $init_args, $lnum, $lname)) }; }
+                                if ($object) { $logger = sub { shift;                 $logger0->($init_args,        $layouter->($formatter->(@_), $init_args, $lnum, $lname)) };
+                                } else {       $logger = sub {                        $logger0->($init_args,        $layouter->($formatter->(@_), $init_args, $lnum, $lname)) }; }
                             }
                         }
                     } else { # no layouter
                         if ($logger0_is_ml) {
                             if ($routine_name_is_ml) {
-                                if ($object) { $logger = sub { shift; my $lvl=shift; $logger0->($init_args, $lvl,              $formatter->(@_)                            ) };
-                                } else {       $logger = sub {        my $lvl=shift; $logger0->($init_args, $lvl,              $formatter->(@_)                            ) }; }
+                                if ($object) { $logger = sub { shift; my $lnum=shift; $logger0->($init_args, $lnum,             $formatter->(@_)                            ) };
+                                } else {       $logger = sub {        my $lnum=shift; $logger0->($init_args, $lnum,             $formatter->(@_)                            ) }; }
                             } else { # routine name not multiple-level
-                                if ($object) { $logger = sub { shift;                $logger0->($init_args, $lnum,             $formatter->(@_)                            ) };
-                                } else {       $logger = sub {                       $logger0->($init_args, $lnum,             $formatter->(@_)                            ) }; }
+                                if ($object) { $logger = sub { shift;                 $logger0->($init_args, $lnum,             $formatter->(@_)                            ) };
+                                } else {       $logger = sub {                        $logger0->($init_args, $lnum,             $formatter->(@_)                            ) }; }
                             }
                         } else { # logger0 not multiple-level
                             if ($routine_name_is_ml) {
                                 if ($object) { $logger = sub { shift; return 0 if Log::ger::Util::numeric_level(shift) > $Current_Level;
-                                                                                     $logger0->($init_args,                    $formatter->(@_)                            ) };
+                                                                                      $logger0->($init_args,                    $formatter->(@_)                            ) };
                                 } else {       $logger = sub {        return 0 if Log::ger::Util::numeric_level(shift) > $Current_Level;
-                                                                                     $logger0->($init_args,                    $formatter->(@_)                            ) }; }
+                                                                                      $logger0->($init_args,                    $formatter->(@_)                            ) }; }
                             } else { # routine name not multiple-level
-                                if ($object) { $logger = sub { shift;                $logger0->($init_args,                    $formatter->(@_)                            ) };
-                                } else {       $logger = sub {                       $logger0->($init_args,                    $formatter->(@_)                            ) }; }
+                                if ($object) { $logger = sub { shift;                 $logger0->($init_args,                    $formatter->(@_)                            ) };
+                                } else {       $logger = sub {                        $logger0->($init_args,                    $formatter->(@_)                            ) }; }
                             }
                         }
                     }
@@ -317,26 +329,27 @@ sub init_target {
                     { # no layouter, just to align
                         if ($logger0_is_ml) {
                             if ($routine_name_is_ml) {
-                                if ($object) { $logger = sub { shift; my $lvl=shift; $logger0->($init_args, $lvl,                           @_                             ) };
-                                } else {       $logger = sub {        my $lvl=shift; $logger0->($init_args, $lvl,                           @_                             ) }; }
+                                if ($object) { $logger = sub { shift; my $lnum=shift; $logger0->($init_args, $lnum,                          @_                             ) };
+                                } else {       $logger = sub {        my $lnum=shift; $logger0->($init_args, $lnum,                          @_                             ) }; }
                             } else { # routine name not multiple-lvl
-                                if ($object) { $logger = sub { shift;                $logger0->($init_args, $lnum,                          @_                             ) };
-                                } else {       $logger = sub {                       $logger0->($init_args, $lnum,                          @_                             ) }; }
+                                if ($object) { $logger = sub { shift;                 $logger0->($init_args, $lnum,                          @_                             ) };
+                                } else {       $logger = sub {                        $logger0->($init_args, $lnum,                          @_                             ) }; }
                             }
                         } else { # logger0 not multiple-level
                             if ($routine_name_is_ml) {
                                 if ($object) { $logger = sub { shift; return 0 if Log::ger::Util::numeric_level(shift) > $Current_Level;
-                                                                                     $logger0->($init_args,                                 @_                             ) };
+                                                                                      $logger0->($init_args,                                 @_                             ) };
                                 } else {       $logger = sub {        return 0 if Log::ger::Util::numeric_level(shift) > $Current_Level;
-                                                                                     $logger0->($init_args,                                 @_                             ) }; }
+                                                                                      $logger0->($init_args,                                 @_                             ) }; }
                             } else {
-                                if ($object) { $logger = sub { shift;                $logger0->($init_args,                                 @_                             ) };
-                                } else {       $logger = sub {                       $logger0->($init_args,                                 @_                             ) }; }
+                                if ($object) { $logger = sub { shift;                 $logger0->($init_args,                                 @_                             ) };
+                                } else {       $logger = sub {                        $logger0->($init_args,                                 @_                             ) }; }
                             }
                         }
                     }
                 }
             }
+          L1:
             my $type = $routine_name_is_ml ?
                 ($object ? 'logml_method' : 'logml_sub') :
                 ($object ? 'log_method' : 'log_sub');
@@ -450,7 +463,7 @@ Log::ger - A lightweight, flexible logging framework
 
 =head1 VERSION
 
-version 0.011
+version 0.012
 
 =head1 SYNOPSIS
 
