@@ -1,7 +1,7 @@
 package Log::ger::Util;
 
-our $DATE = '2017-06-30'; # DATE
-our $VERSION = '0.011'; # VERSION
+our $DATE = '2017-07-02'; # DATE
+our $VERSION = '0.012'; # VERSION
 
 use strict;
 use warnings;
@@ -153,13 +153,17 @@ sub set_plugin {
 
     my %args = @_;
 
-    my $prefix = $args{prefix} || 'Log::ger::Plugin::';
-
-    my $mod = $args{name};
-    $mod = $prefix . $mod unless index($mod, $prefix) == 0;
-    (my $mod_pm = "$mod.pm") =~ s!::!/!g;
-    require $mod_pm;
-    my $hooks = &{"$mod\::get_hooks"}(%{ $args{conf} || {} });
+    my $hooks;
+    if ($args{hooks}) {
+        $hooks = $args{hooks};
+    } else {
+        my $prefix = $args{prefix} || 'Log::ger::Plugin::';
+        my $mod = $args{name};
+        $mod = $prefix . $mod unless index($mod, $prefix) == 0;
+        (my $mod_pm = "$mod.pm") =~ s!::!/!g;
+        require $mod_pm;
+        $hooks = &{"$mod\::get_hooks"}(%{ $args{conf} || {} });
+    }
 
     for my $phase (keys %$hooks) {
         my $hook = $hooks->{$phase};
@@ -238,7 +242,7 @@ Log::ger::Util - Utility routines for Log::ger
 
 =head1 VERSION
 
-version 0.011
+version 0.012
 
 =head1 DESCRIPTION
 
